@@ -1,7 +1,7 @@
 package LWP::UserAgent::ProgressAny;
 
 our $DATE = '2015-01-22'; # DATE
-our $VERSION = '0.04'; # VERSION
+our $VERSION = '0.05'; # VERSION
 
 use 5.010001;
 use strict;
@@ -38,43 +38,39 @@ sub __add_handlers {
                 $progress->target($cl);
             }
         }
-        my $new_pos = $progress->pos() + length($data);
-        $progress->update(
-            pos => $new_pos,
-            message => sub {
-                my @msg = (
-                    "Downloading ",
-                    $resp->{_request}{_uri},
-                    " ",
-                    format_metric($new_pos),
-                );
-                if ($progress->target) {
-                    push @msg, "/", format_metric($progress->target);
-                }
-                # XXX show speed
+        my $new_pos = $progress->pos + length($data);
+        my $target  = $progress->target;
+
+        if ($target && $new_pos >= $target) {
+            $progress->finish;
+
+            # cleanup so the number of tasks can be kept low. XXX we should do
+            # this via API.
+            no warnings 'once';
+            delete $Progress::Any::indicators{$task};
+            delete $ua->{_pa_data}{set_target}{"$resp"};
+        } else {
+            $progress->update(
+                pos => $new_pos,
+                message => sub {
+                    my @msg = (
+                        "Downloading ",
+                        $resp->{_request}{_uri},
+                        " ",
+                        format_metric($new_pos),
+                    );
+                    if ($progress->target) {
+                        push @msg, "/", format_metric($progress->target);
+                    }
+                    # XXX show speed
                 join "", @msg;
-            },
-        );
+                },
+            );
+        }
 
         # so we are called again for the next chunk
         1;
     });
-
-    $ua->add_handler(response_done => sub {
-        my ($resp, $ua, $h) = @_;
-
-        my $task = __get_task_name($resp);
-
-        my $progress = Progress::Any->get_indicator(task=>$task);
-        $progress->finish;
-
-        # cleanup so the number of tasks can be kept low. XXX we should do this
-        # via API.
-        no warnings 'once';
-        delete $Progress::Any::indicators{$task};
-
-        delete $ua->{_pa_data}{set_target}{"$resp"};
-    });
 }
 
 sub new {
@@ -100,7 +96,7 @@ LWP::UserAgent::ProgressAny - See progress for your LWP::UserAgent requests
 
 =head1 VERSION
 
-This document describes version 0.04 of LWP::UserAgent::ProgressAny (from Perl distribution LWP-UserAgent-ProgressAny), released on 2015-01-22.
+This document describes version 0.05 of LWP::UserAgent::ProgressAny (from Perl distribution LWP-UserAgent-ProgressAny), released on 2015-01-22.
 
 =head1 SYNOPSIS
 
@@ -140,11 +136,14 @@ Sample output:
 
 L<LWP::UserAgent::ProgressBar> (LU::PB) is a similar module. It uses
 L<Term::ProgressBar> to display progress bar and introduces two new methods:
-C<get_with_progress> and C<put_with_progress>. Compared to
+C<get_with_progress> and C<post_with_progress>. Compared to
 LWP::UserAgent::ProgressAny (LU::PA): LU::PA uses L<Progress::Any> so you can
 get progress notification via means other than terminal progress bar simply by
 choosing another progress output. LU::PA is also more transparent, you don't
-have to use a different method to do requests.
+have to use a different method to do requests. Lastly, LU::PA can be used with
+standard LWP::UserAgent or its other subclasses.
+
+L<HTTP::Tiny::ProgressAny>
 
 =head1 HOMEPAGE
 
